<?php

namespace Drupal\gtfs\Controller;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Url;
use Drupal\gtfs\Entity\GTFSObjectInterface;

/**
 * Class AgencyController.
 *
 *  Returns responses for GTFS Agency routes.
 */
class AgencyController extends ControllerBase implements ContainerInjectionInterface {

  /**
   * Displays a GTFS Agency  revision.
   *
   * @param int $agency_revision
   *   The GTFS Agency revision ID.
   *
   * @return array
   *   An array suitable for drupal_render().
   */
  public function revisionShow($agency_revision) {
    $agency = $this->entityManager()->getStorage('gtfs_agency')->loadRevision($agency_revision);
    $view_builder = $this->entityManager()->getViewBuilder('gtfs_agency');

    return $view_builder->view($agency);
  }

  /**
   * Page title callback for a GTFS Agency revision.
   *
   * @param int $agency_revision
   *   The GTFS Agency revision ID.
   *
   * @return string
   *   The page title.
   */
  public function revisionPageTitle($agency_revision) {
    $agency = $this->entityManager()->getStorage('gtfs_agency')->loadRevision($agency_revision);
    return $this->t('Revision of %title from %date', ['%title' => $agency->label(), '%date' => format_date($agency->getRevisionCreationTime())]);
  }

  /**
   * Generates an overview table of older revisions of a GTFS Agency .
   *
   * @param \Drupal\gtfs\Entity\GTFSObjectInterface $agency
   *   A GTFS Agency object.
   *
   * @return array
   *   An array as expected by drupal_render().
   */
  public function revisionOverview(GTFSObjectInterface $agency) {
    $account = $this->currentUser();
    $langcode = $agency->language()->getId();
    $langname = $agency->language()->getName();
    $languages = $agency->getTranslationLanguages();
    $has_translations = (count($languages) > 1);
    $agency_storage = $this->entityManager()->getStorage('gtfs_agency');

    $build['#title'] = $has_translations ? $this->t('@langname revisions for %title', ['@langname' => $langname, '%title' => $agency->label()]) : $this->t('Revisions for %title', ['%title' => $agency->label()]);
    $header = [$this->t('Revision'), $this->t('Operations')];

    $revert_permission = (($account->hasPermission("revert all gtfs agency revisions") || $account->hasPermission('administer gtfs agency entities')));
    $delete_permission = (($account->hasPermission("delete all gtfs agency revisions") || $account->hasPermission('administer gtfs agency entities')));

    $rows = [];

    $vids = $agency_storage->revisionIds($agency);

    $latest_revision = TRUE;

    foreach (array_reverse($vids) as $vid) {
      /** @var \Drupal\gtfs\Entity\GTFSObjectInterface $revision */
      $revision = $agency_storage->loadRevision($vid);
      // Only show revisions that are affected by the language that is being
      // displayed.
      if ($revision->hasTranslation($langcode) && $revision->getTranslation($langcode)->isRevisionTranslationAffected()) {
        $username = [
          '#theme' => 'username',
          '#account' => $revision->getRevisionUser(),
        ];

        // Use revision link to link to revisions that are not active.
        $date = \Drupal::service('date.formatter')->format($revision->getRevisionCreationTime(), 'short');
        if ($vid != $agency->getRevisionId()) {
          $link = $this->l($date, new Url('entity.gtfs_agency.revision', ['gtfs_agency' => $agency->id(), 'gtfs_agency_revision' => $vid]));
        }
        else {
          $link = $agency->link($date);
        }

        $row = [];
        $column = [
          'data' => [
            '#type' => 'inline_template',
            '#template' => '{% trans %}{{ date }} by {{ username }}{% endtrans %}{% if message %}<p class="revision-log">{{ message }}</p>{% endif %}',
            '#context' => [
              'date' => $link,
              'username' => \Drupal::service('renderer')->renderPlain($username),
              'message' => ['#markup' => $revision->getRevisionLogMessage(), '#allowed_tags' => Xss::getHtmlTagList()],
            ],
          ],
        ];
        $row[] = $column;

        if ($latest_revision) {
          $row[] = [
            'data' => [
              '#prefix' => '<em>',
              '#markup' => $this->t('Current revision'),
              '#suffix' => '</em>',
            ],
          ];
          foreach ($row as &$current) {
            $current['class'] = ['revision-current'];
          }
          $latest_revision = FALSE;
        }
        else {
          $links = [];
          if ($revert_permission) {
            $links['revert'] = [
              'title' => $this->t('Revert'),
              'url' => $has_translations ?
              Url::fromRoute('entity.gtfs_agency.translation_revert', ['gtfs_agency' => $agency->id(), 'gtfs_agency_revision' => $vid, 'langcode' => $langcode]) :
              Url::fromRoute('entity.gtfs_agency.revision_revert', ['gtfs_agency' => $agency->id(), 'gtfs_agency_revision' => $vid]),
            ];
          }

          if ($delete_permission) {
            $links['delete'] = [
              'title' => $this->t('Delete'),
              'url' => Url::fromRoute('entity.gtfs_agency.revision_delete', ['gtfs_agency' => $agency->id(), 'gtfs_agency_revision' => $vid]),
            ];
          }

          $row[] = [
            'data' => [
              '#type' => 'operations',
              '#links' => $links,
            ],
          ];
        }

        $rows[] = $row;
      }
    }

    $build['gtfs_agency_revisions_table'] = [
      '#theme' => 'table',
      '#rows' => $rows,
      '#header' => $header,
    ];

    return $build;
  }

}
