<?php

namespace Drupal\gtfs\Entity;

use Yasumi\Yasumi;

use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Entity\EntityTypeInterface;

/**
 * Defines the GTFS Calendar Date entity.
 *
 * @ingroup gtfs
 *
 * @ContentEntityType(
 *   id = "gtfs_calendar_date",
 *   label = @Translation("GTFS Calendar Date"),
 *   handlers = {
 *     "storage" = "Drupal\gtfs\CalendarDateStorage",
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\gtfs\CalendarDateListBuilder",
 *     "views_data" = "Drupal\views\EntityViewsData",
 *     "translation" = "Drupal\content_translation\ContentTranslationHandler",
 *
 *     "form" = {
 *       "default" = "Drupal\gtfs\Form\CalendarDateForm",
 *       "add" = "Drupal\gtfs\Form\CalendarDateForm",
 *       "edit" = "Drupal\gtfs\Form\CalendarDateForm",
 *       "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm",
 *     },
 *     "access" = "Drupal\gtfs\CalendarDateAccessControlHandler",
 *     "route_provider" = {
 *       "html" = "Drupal\gtfs\CalendarDateHtmlRouteProvider",
 *     },
 *   },
 *   base_table = "gtfs_calendar_date",
 *   data_table = "gtfs_calendar_date_field_data",
 *   revision_table = "gtfs_calendar_date_revision",
 *   revision_data_table = "gtfs_calendar_date_field_revision",
 *   translatable = TRUE,
 *   admin_permission = "administer gtfs calendar_date entities",
 *   entity_keys = {
 *     "id" = "id",
 *     "revision" = "vid",
 *     "label" = "date",
 *     "uuid" = "uuid",
 *     "uid" = "user_id",
 *     "langcode" = "langcode",
 *     "status" = "status",
 *   },
 *   links = {
 *     "canonical" = "/admin/gtfs/calendar_date/{gtfs_calendar_date}",
 *     "add-form" = "/admin/gtfs/calendar_date/add",
 *     "edit-form" = "/admin/gtfs/calendar_date/{gtfs_calendar_date}/edit",
 *     "delete-form" = "/admin/gtfs/calendar_date/{gtfs_calendar_date}/delete",
 *     "version-history" = "/admin/gtfs/calendar_date/{gtfs_calendar_date}/revisions",
 *     "revision" = "/admin/gtfs/calendar_date/{gtfs_calendar_date}/revisions/{gtfs_calendar_date_revision}/view",
 *     "revision_revert" = "/admin/gtfs/calendar_date/{gtfs_calendar_date}/revisions/{gtfs_calendar_date_revision}/revert",
 *     "revision_delete" = "/admin/gtfs/calendar_date/{gtfs_calendar_date}/revisions/{gtfs_calendar_date_revision}/delete",
 *     "translation_revert" = "/admin/gtfs/calendar_date/{gtfs_calendar_date}/revisions/{gtfs_calendar_date_revision}/revert/{langcode}",
 *     "collection" = "/admin/gtfs/calendar_date",
 *   },
 *   field_ui_base_route = "gtfs_calendar_date.settings"
 * )
 */
class CalendarDate extends GTFSEntityBase {

  const GTFS_FORMAT = 'Ymd';

  const DB_FORMAT = 'Y-m-d';

  const EXCEPTION_TYPE_ADDED = '1';

  const EXCEPTION_TYPE_REMOVED = '2';

  public static function schema() {
    return [
      'fields' => [
        'service_id' => ['type' => 'varchar', 'length' => 128, 'not null' => TRUE],
        'date' => ['type' => 'varchar', 'length' => 8, 'not null' => TRUE],
        'exception_type' => ['type' => 'int', 'length' => 2],
      ],
      'primary key' => ['service_id', 'date'],
      'foreign keys' => [
        'service' => [
          'table' => 'gtfs_service',
          'columns' => ['service_id' => 'id']
        ]
      ]
    ];
  }

  public function service() {
    $service_id = $this->get('service_id')->target_id;
    return Service::load($service_id);
  }

  public function dateName() {
    $date = \DateTime::createFromFormat(self::DB_FORMAT, $this->get('date')->value);
    $provider = Yasumi::create('USA', (int) date('Y', $date->getTimestamp()));
    $datename = null;
    foreach($provider->getHolidays() as $holiday) {
      if($holiday->format(self::GTFS_FORMAT) === $date->format(self::GTFS_FORMAT)) {
        $datename = $holiday->getName();
      }
    }
    if(!$datename) {
      $datename = $date->format('F jS, Y');
    }
    return $datename;
  }

  public function toGTFSObject() {
    $return = parent::toGTFSObject();
    $return['date'] = \DateTime::createFromFormat(self::DB_FORMAT, $return['date'])->format(self::GTFS_FORMAT);
    $return['name'] = $this->dateName();
    unset($return['self']);
    return $return;
  }

  /**
   * {@inheritdoc}
   */
  public function getName() {
    $action = $this->get('exception_type')->value == self::EXCEPTION_TYPE_ADDED ? 'Added' : 'Removed';
    $service = $this->service()->getName();
    $datename = $this->dateName();
    return "{$action} {$service} on {$datename}";
  }

  /**
   * {@inheritdoc}
   */
  public function setName($name) {
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['service_id'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Service ID'))
      ->setDescription(t('The service_id contains an ID that uniquely identifies a set of dates when a service exception is available for one or more routes. Each (service_id, date) pair can only appear once in calendar_dates.txt. If the a service_id value appears in both the calendar.txt and calendar_dates.txt files, the information in calendar_dates.txt modifies the service information specified in calendar.txt. This field is referenced by the trips.txt file.'))
      ->setRevisionable(TRUE)
      ->setSettings([
         'target_type' => 'gtfs_service',
         'default_value' => 0,
         'handler' => 'default'
      ])
      ->setDisplayOptions('form', [
         'type' => 'entity_reference_autocomplete',
         'weight' => -3,
         'settings' => [
       'match_operator' => 'CONTAINS',
       'size' => '60',
       'autocomplete_type' => 'tags',
       'placeholder' => '',
         ],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setRequired(TRUE);

    $fields['date'] = BaseFieldDefinition::create('datetime')
      ->setLabel(t('Date'))
      ->setDescription(t('The date field specifies a particular date when service availability is different than the norm. You can use the exception_type field to indicate whether service is available on the specified date.'))
      ->setSetting('datetime_type', 'date')
      ->setRequired(TRUE)
      ->setRevisionable(TRUE)
      ->setDisplayOptions('form', [
        'type' => 'datetime_default',
        'weight' => -3,
      ])
      ->setDisplayOptions('view', array(
        'label' => 'above',
        'type' => 'string',
        'weight' => -4,
      ))
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['exception_type'] = BaseFieldDefinition::create('list_string')
      ->setLabel(t('Exception Type'))
      ->setDescription(t('The exception_type indicates whether service is available on the date specified in the date field.'))
      ->setRevisionable(TRUE)
      ->setDefaultValue('1')
      ->setSettings([
        'allowed_values' => [
           self::EXCEPTION_TYPE_ADDED => 'Added',
           self::EXCEPTION_TYPE_REMOVED => 'Removed'
        ]
      ])
      ->setDisplayOptions('form', [
        'type' => 'options_buttons',
        'weight' => -3
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setRequired(TRUE);

    return $fields;
  }

}
