<?php

namespace Drupal\gtfs\Entity\Feed;

use Drupal\gtfs\Entity\FeedInterface;
use Drupal\Core\Archiver\Zip;

/**
 * Provides the ability to download from a GTFS feed.
 */
class Downloader {

  const MSG_FILE_DOWNLOADING = 'Downloading @filename';
  const MSG_FILE_EXISTS = '@filename has already been downloaded. Skipping.';
  const MSG_FILE_DOWNLOADED = '@filename downloaded to private files.';
  const MSG_ERR_FILE_DOWNLOAD = 'File could not be downloaded. Check your configuration and log messages';
  const MSG_FILE_EXTRACTED = 'Extracted @directory and removed @filename';

  /**
   * The feed which is being downloaded.
   *
   * @var \Drupal\gtfs\Entity\FeedInterface
   */
  private $feed;

  /**
   * The file that was downloaded.
   *
   * @var string
   */
  private $downloadedFile = '';

  /**
   * The location of the files after download and extraction.
   *
   * @var string
   */
  private $extractedDirectory = '';

  /**
   * Constructs a new importer instance.
   *
   * @param \Drupal\gtfs\FeedInterface $feed
   *   A Feed object.
   */
  public function __construct(FeedInterface $feed) {
    $this->feed = $feed;
    if (file_exists($this->getFileDownloadLocation())) {
      $this->downloadedFile = $this->getFileDownloadLocation();
    }
    if (!is_dir($this->getExtractedFileDirectory())) {
      mkdir($this->getExtractedFileDirectory());
    }
    $this->extractedDirectory = $this->getExtractedFileDirectory();
  }

  /**
   *
   */
  public function batch(): array {
    return [
      'title' => t(static::MSG_FILE_DOWNLOADING, [
        '@filename' => $this->getFeedFilename(),
      ]),
      'operations' => [
        [[$this, 'download'], []],
        [[$this, 'extract'], []],
      ],
      'finished' => [$this, 'batchFinished'],
    ];
  }

  /**
   *
   */
  public function batchFinished() {}

  /**
   * Downloads the GTFS feed file.
   *
   * @param array $context
   *   The batch context array.
   */
  public function download(array &$context) {
    if ($this->downloadedFile) {
      $context['message'] = t(static::MSG_FILE_EXISTS, ['@filename' => $this->downloadedFile]);
      return TRUE;
    }

    $url = $this->getFeedUrl();
    $destination = $this->getFileDownloadLocation();
    $managed = FALSE;
    $replace = FILE_EXISTS_REPLACE;

    try {
      $this->downloadedFile = system_retrieve_file($url, $destination, $managed, $replace);
    } catch (\Exception $e) {
      \Drupal::logger('gtfs')->error($e->getMessage());
    }

    if (!$this->downloadedFile) {
      // An error will be thrown by system_retrieve_file.
      return FALSE;
    }

    $context['message'] = t(static::MSG_FILE_DOWNLOADED, ['@filename' => $destination]);
    return TRUE;
  }

  /**
   *
   */
  public function extract(array &$context) {
    if (!$this->downloadedFile) {
      $downloaded = $this->download($context);
      if (!$downloaded) {
        throw new \Exception(static::MSG_ERR_FILE_DOWNLOAD);
      }
    }

    $zip = new Zip($this->downloadedFile);
    $extracted = $zip->extract($this->extractedDirectory);
    unlink($this->downloadedFile);
    $context['message'] = t(static::MSG_FILE_EXTRACTED, ['@directory' => $this->extractedDirectory, '@filename' => $this->downloadedFile]);
    $this->downloadedFile = '';
  }

  /**
   *
   */
  public function removeFile($filename) {
    $filepath = "{$this->extractedDirectory}/{$filename}.txt";
    if (is_file($filepath)) {
      unlink($filepath);
    }
  }

  /**
   *
   */
  public function removeDirectory() {
    $files = file_scan_directory($this->extractedDirectory, '/.*/');
    foreach ($files as $file) {
      unlink($file->uri);
    }
    rmdir($this->extractedDirectory);
  }

  /**
   *
   */
  private function getFeedUrl(): string {
    return $this->feed->get('feed_url')->value;
  }

  /**
   *
   */
  private function getFeedFilename(): string {
    return basename($this->getFeedUrl());
  }

  /**
   *
   */
  private function getFileDownloadLocation(): string {
    $base = drupal_realpath('private://');
    $filename = $this->getFeedFilename();
    return "{$base}/{$filename}";
  }

  /**
   *
   */
  public function getExtractedFileDirectory(): string {
    $base = drupal_realpath('private://');
    $dirname = str_replace('.zip', '', $this->getFeedFilename());
    return "{$base}/{$dirname}";
  }

}
