<?php

namespace Drupal\gtfs\Entity\Feed;

use Drupal\Component\Uuid\Php as UUID;
use PDO;
use SplFileObject;
use Drupal\gtfs\Entity\Feed;
use Drupal\gtfs\Entity\FeedInterface;
use Drupal\core\Url;
use Drupal;
use Drupal\Core\Database\Database;

/**
 *
 */
class Importer {

  public $id;

  private $downloader;

  public $filesImporting = [];

  const BATCH_SIZE = 100;
  const SPL_FLAGS = SplFileObject::DROP_NEW_LINE | SplFileObject::READ_CSV | SplFileObject::READ_AHEAD | SplFileObject::SKIP_EMPTY;

  /**
   * Constructs a new instance of the importer based on the database.
   *
   * @param string|int $importId
   *   The id of the cswa_feed_imports row.
   */
  public function __construct($importId) {
    $this->id = $importId;
  }

  /**
   * Overloads the __get function to get values from database.
   */
  public function __get($name) {
    if ($name === 'feed') {
      return $this->getFeed();
    }
    $dbentry = \Drupal::database()->query(
      'SELECT *
       FROM {gtfs_feed_imports}
       WHERE `id` = :id',
       [':id' => $this->id]
    )->fetch(PDO::FETCH_OBJ);
    $dbentry->files = unserialize($dbentry->files);
    $dbentry->completed = unserialize($dbentry->completed);
    if (isset($dbentry->{$name})) {
      return $dbentry->{$name};
    }
    $trace = debug_backtrace();
    trigger_error(
      'Undefined property via __get(): ' . $name .
      ' in ' . $trace[0]['file'] .
      ' on line ' . $trace[0]['line'],
      E_USER_NOTICE);
  }

  /**
   * Overloads the set function to set the completed value in database.
   */
  public function __set($name, $value) {
    if ($name === 'completed') {
      Drupal::database()
        ->update('gtfs_feed_imports')
        ->fields([
          'completed' => serialize($value),
        ])
        ->condition('id', $this->id, '=')
        ->execute();
    }
  }

  /**
   * Given the location of a file,
   * counts the number of records (lines - 1)
   */
  public static function countRecordsInFile($filepath) {
    $splFile = new SplFileObject($filepath);
    $splFile->setFlags(static::SPL_FLAGS);
    $splFile->seek($splFile->getSize());
    $records_in_file = number_format($splFile->key() - 1);
    return $records_in_file;
  }

  /**
   * Returns the feed from which entities are being imported.
   *
   * @return \Drupal\gtfs\Entity\FeedInterface
   *   The feed associated with the import.
   */
  public function getFeed(): FeedInterface {
    $feedId = Drupal::database()->query(
      'SELECT `feed_id`
       FROM {gtfs_feed_imports}
       WHERE `id` = :id',
      [':id' => $this->id]
    )->fetchColumn();
    return Feed::load($feedId);
  }

  /**
   * Gets the order of files to import.
   *
   * This function returns the order of files
   * to import, as specified by this and any
   * other extending modules.
   *
   * e.g. HOOK_gtfs_import_order_alter
   *
   * @return array
   *   An array of filenames, in order.
   */
  public static function getImportOrder(): array {
    $import_order = [];
    Drupal::moduleHandler()->alter('gtfs_import_order', $import_order);
    return $import_order;
  }

  /**
   * Gets the map of which entities handle which files.
   *
   * This function returns class handlers for each file
   * in the GTFS file. Classes must implement the
   * \Drupal\gtfs\Entity\GTFSObjectInterface. This list
   * may be modified by other modules.
   *
   * e.g. HOOK_gtfs_import_entities_alter
   *
   * @return array
   *   An array keyed by filename, valued by class
   */
  public static function getFileEntityMap(): array {
    $file_entity_map = [];
    Drupal::moduleHandler()->alter('gtfs_import_entities', $file_entity_map);
    return $file_entity_map;
  }

  /**
   * Gets a feed reference object to attach to imported entities.
   *
   * @return array
   *   The feed reference.
   */
  public function getFeedReference(): array {
    return [
      'target_id' => $this->feed->id(),
      'target_revision_id' => $this->feed->getRevisionId(),
    ];
  }

  /**
   * Builds the batch that creates temporary tables.
   *
   * Before inserting into the database, the importer
   * creates a temporary table for each file and in-
   * serts the data into them.
   *
   * @return array
   *   A Drupal batch description
   *
   * @see https://www.drupal.org/docs/7/api/batch-api
   */
  public function batchPrepare(): array {
    $operations = [];

    $file_entity_map = static::getFileEntityMap();
    $downloader = new Downloader($this->feed);

    $files = $this->files;

    foreach ($files as $file) {
      // $class = $file_entity_map[$file];.
      $filepath = "{$downloader->getExtractedFileDirectory()}/{$file}.txt";
      // In case the file doesn't end in a hard return.
      passthru('sed -i -e "$a\" ' . $filepath);
      $operations[] = [
        [$this, 'tempTableCreate'],
        [$file],
      ];
      $operations[] = [
        [$this, 'tempTableLoad'],
        [$file],
      ];
      $operations[] = [
        [$downloader, 'removeFile'],
        [$file],
      ];
      $operations[] = [
        [$this, 'tempTableHash'],
        [$file],
      ];
      $operations[] = [
        [$this, 'tempTableIndexHash'],
        [$file],
      ];
    }

    $unimportedFiles = array_diff(array_keys($file_entity_map), $files);
    foreach ($unimportedFiles as $file) {
      $operations[] = [
        [$downloader, 'removeFile'],
        [$file],
      ];
    }

    $operations[] = [
      [$downloader, 'removeDirectory'],
      [],
    ];

    $batch = [
      'title' => t('Preparing entities'),
      'operations' => $operations,
      'finished' => [$this, 'batchPrepareFinished'],
      'next_url' => Url::fromRoute('entity.gtfs_feed.import_form', [
        'gtfs_feed' => $this->feed->id(),
        'step' => 'transform',
        'class' => base64_encode($file_entity_map[reset($files)]),
      ]),
    ];

    return $batch;
  }

  /**
   * Creates a temporary table given a GTFS file.
   *
   * @param string $file
   *   A filename in the import.
   * @param array &$context
   *   The batch context.
   *
   * @see https://www.drupal.org/docs/7/api/batch-api
   */
  public function tempTableCreate(string $file, array &$context) {
    $file_entity_map = static::getFileEntityMap();
    $class = $file_entity_map[$file];
    $temp_table_name = \Drupal::service('entity_type.repository')->getEntityTypeFromClass($class) . '_temp';
    $feedReference = $this->getFeedReference();
    $db = Drupal::database();
    $db_schema = $db->schema();
    if ($db_schema->tableExists($temp_table_name)) {
      $db_schema->dropTable($temp_table_name);
    }

    $schema = $class::schema();
    $schema['fields'] = array_merge($schema['fields'], [
      'hash' => ['type' => 'varchar', 'length' => 32],
      'index_hash' => ['type' => 'varchar', 'length' => 32],
      'feed_reference__target_id' => [
        'type' => 'int',
        'length' => 11,
        'default' => $feedReference['target_id']
      ],
      'feed_reference__target_revision_id' => [
        'type' => 'int',
        'length' => 11,
        'default' => $feedReference['target_revision_id']
      ],
      'langcode' => ['type' => 'varchar', 'length' => 12, 'default' => 'en'],
      'status' => ['type' => 'int', 'length' => 4, 'default' => 1],
      'revision_translation_affected' => ['type' => 'int', 'length' => 4, 'default' => 1],
      'default_langcode' => ['type' => 'int', 'length' => 4, 'default' => 1],
      'user_id' => ['type' => 'varchar', 'length' => 11, 'default' => Drupal::currentUser()->id()],
    ]);

    // Null values break hashing.
    foreach ($schema['fields'] as &$field) {
      $field['not null'] = TRUE;
    }

    $db_schema->createTable($temp_table_name, $schema);
    $context['message'] = t('Created @tablename table', ['@tablename' => $temp_table_name]);
  }

  /**
   * Loads data into a temporary table from a file.
   *
   * @param string $file
   *   A filename in the import.
   * @param array &$context
   *   The batch context.
   *
   * @see https://www.drupal.org/docs/7/api/batch-api
   */
  public function tempTableLoad(string $file, array &$context) {
    $file_entity_map = static::getFileEntityMap();
    $class = $file_entity_map[$file];
    $allowed_columns = array_keys($class::schema()['fields']);
    $downloader = new Downloader($this->feed);
    $filepath = "{$downloader->getExtractedFileDirectory()}/{$file}.txt";
    $columns = explode(',', fgets(fopen($filepath, 'r')));
    $columns = array_map('trim', $columns);
    $columns = array_map(function ($column) use ($allowed_columns) {
      return in_array($column, $allowed_columns) ? $column : '@dummy';
    }, $columns);
    $columns = implode(',', $columns);

    $temp_table_name = \Drupal::service('entity_type.repository')->getEntityTypeFromClass($class) . '_temp';

    $connection = Drupal::database()->getConnectionOptions();
    $connection['pdo'][PDO::MYSQL_ATTR_LOCAL_INFILE] = 1;
    Database::addConnectionInfo('extra', 'default', $connection);
    Database::setActiveConnection('extra');

    $terminator = EOLDetector::detectFileEOL($filepath);

    Database::getConnection()->query(
      "LOAD DATA LOCAL INFILE '{$filepath}'
       INTO TABLE {$temp_table_name}
       FIELDS TERMINATED BY ','
       LINES TERMINATED BY '{$terminator}'
       IGNORE 1 LINES
       ({$columns});"
    )->execute();

    // Set connection back to default;.
    Database::setActiveConnection();

    $context['message'] = t('Imported @filepath into @tablename', ['@filepath' => str_replace(drupal_realpath('private://') . '/', '', $filepath), '@tablename' => $temp_table_name]);
  }

  /**
   * Hashes a temporary table of entities.
   *
   * Hashing an object's data allows us to see if it has changed
   * since last import.
   *
   * @param string $file
   *   A filename in the import.
   * @param array &$context
   *   The batch context.
   *
   * @see https://www.drupal.org/docs/7/api/batch-api
   */
  public function tempTablehash(string $file, array &$context) {
    $file_entity_map = static::getFileEntityMap();
    $class = $file_entity_map[$file];

    $temp_table_name = \Drupal::service('entity_type.repository')->getEntityTypeFromClass($class) . '_temp';

    $schema = $class::schema();

    $cast_fields = [];
    foreach (array_keys($schema['fields']) as $key) {
      $cast_fields[] = "CAST({$key} as CHAR)";
    }

    $cast_fields = implode(',', $cast_fields);

    Drupal::database()->query(
      "UPDATE `{$temp_table_name}`
       SET `hash` = CRC32(CONCAT({$cast_fields}));"
    );

    $context['message'] = t('Hashed @tablename', ['@tablename' => $temp_table_name]);
  }

  /**
   * Index hashes data a temporary table of entities.
   *
   * Index hashing an object allows us to identify which objects
   * persist even if data has changed.
   *
   * @param string $file
   *   A filename in the import.
   * @param array &$context
   *   The batch context.
   *
   * @see https://www.drupal.org/docs/7/api/batch-api
   */
  public function tempTableIndexHash(string $file, array &$context) {
    $file_entity_map = static::getFileEntityMap();
    $class = $file_entity_map[$file];

    $temp_table_name = \Drupal::service('entity_type.repository')->getEntityTypeFromClass($class) . '_temp';

    $schema = $class::schema();

    $index_hash_fields = implode(',', array_merge($schema['primary key'], ['feed_reference__target_id']));

    Drupal::database()->query(
      "UPDATE {{$temp_table_name}}
       SET `index_hash` = CRC32(CONCAT({$index_hash_fields}));"
    );

    $context['message'] = t('Hashed index of @tablename', ['@tablename' => $temp_table_name]);
  }

  /**
   *
   */
  public function batchPrepareFinished($success, $results, $operations) {}

  /**
   * Builds a batch given a single entity type.
   *
   * @param string $class
   *   The class for which to build the batch.
   *
   * @return array
   *   A Drupal batch description.
   *
   * @see https://www.drupal.org/docs/7/api/batch-api
   */
  public function batchTransform(string $class): array {
    $schema = $class::schema();
    $entity_type = \Drupal::service('entity_type.repository')->getEntityTypeFromClass($class);
    $db = Drupal::database();
    $operations = [];
    $feedReference = $this->getFeedReference();
    $file_entity_map = static::getFileEntityMap();

    $new_count = $db->query(
      "SELECT count(t.index_hash)
       FROM {{$entity_type}_temp} `t`
        WHERE `t`.`index_hash` NOT IN (
          SELECT `fd`.`index_hash`
          FROM {{$entity_type}_field_data} `fd`
          WHERE `fd`.`feed_reference__target_id` = :frti
        )",
      [':frti' => $feedReference['target_id']]
    )->fetchColumn();


    $offset = 0;

    while ($offset < $new_count) {
      $operations[] = [
        [$this, 'transformInsertNew'],
        [$class],
      ];
      $offset += self::BATCH_SIZE;
    }


    $updated_count = $db->query(
      "SELECT count(fd.index_hash)
       FROM {{$entity_type}_field_data} `fd`
       WHERE `fd`.`feed_reference__target_id` = :frti
       AND `fd`.`index_hash` IN (
         SELECT `t`.`index_hash`
         FROM {{$entity_type}_temp} `t`
       ) AND `fd`.`hash` NOT IN (
         SELECT `t`.`hash`
         FROM {{$entity_type}_temp} `t`
       )",
     [':frti' => $feedReference['target_id']]
    )->fetchColumn();


    while ($offset < $updated_count) {
      $operations[] = [
        [$this, 'transformUpdateExisting'],
        [$class],
      ];
      $offset += self::BATCH_SIZE;
    }


    $deleted_count = $db->query(
      "SELECT count(fd.index_hash)
       FROM {{$entity_type}_field_data} `fd`
        WHERE `fd`.`index_hash` NOT IN (
          SELECT `t`.`index_hash`
          FROM {{$entity_type}_temp} `t`
        ) AND `fd`.`feed_reference__target_id` = :frti",
      [':frti' => $feedReference['target_id']]
    )->fetchColumn();


    $offset = 0;

    while ($offset < $deleted_count) {
      $operations[] = [
        [$this, 'transformDeleteOld'],
        [$class],
      ];
      $offset += self::BATCH_SIZE;
    }


    $next_class = NULL;

    foreach ($this->files as $index => $file) {
      if (
        $file_entity_map[$file] == $class
        && isset($this->files[$index + 1])
      ) {
        $next_class = $file_entity_map[$this->files[$index + 1]];
      }
    }

    if ($next_class) {
      $urlParameters = [
        'gtfs_feed' => $this->feed->id(),
        'step' => 'transform',
        'class' => base64_encode($next_class),
      ];
    }
    else {
      $urlParameters = [
        'gtfs_feed' => $this->feed->id(),
        'step' => 'cleanup',
      ];
    }

    return [
      'title' => t('Transforming and Inserting Data'),
      'operations' => $operations,
      'finished' => [$this, 'batchTransformFinished'],
      'next_url' => Url::fromRoute('entity.gtfs_feed.import_form', $urlParameters),
    ];
  }

  /**
   * Transforms and inserts a batch of entities.
   *
   * @param string $class
   *   The class from which to insert entities.
   * @param array $context
   *   The batch context.
   *
   * @see https://www.drupal.org/docs/7/api/batch-api
   */
  public function transformInsertNew(string $class, array &$context) {
    $schema = $class::schema();
    $entity_type = \Drupal::service('entity_type.repository')->getEntityTypeFromClass($class);
    $limit = self::BATCH_SIZE;
    $feedReference = $this->getFeedReference();
    $count = 0;
    $key_cache = [];

    if (isset($schema['foreign keys'])) {
      foreach ($schema['foreign keys'] as $specification) {
        foreach ($specification['columns'] as $source_field => $destination_field) {
          $key_cache[$specification['table']][$source_field] = [];
        }
      }
    }

    $db = Drupal::database();

    $records = $db->query(
      "SELECT `t`.*
       FROM `{$entity_type}_temp` `t`
       WHERE `t`.`index_hash` NOT IN (
         SELECT `fd`.`index_hash`
         FROM `{$entity_type}_field_data` `fd`
       )
       LIMIT {$limit}"
    )->fetchAll(PDO::FETCH_ASSOC);

    if (empty($records)) {
      return;
    }

    Drupal::lock()->acquire('gtfs_lock');

    $base_columns = ['id', 'vid', 'uuid', 'langcode'];
    $revision_columns = [
      'id',
      'vid',
      'langcode',
      'revision_default',
      'revision_user',
      'revision_created',
      'revision_log_message',
    ];
    $data_revision_columns = array_merge(array_keys($records[0]), ['id', 'vid']);
    $data_columns = array_merge($data_revision_columns, ['changed', 'created']);

    $base_insert_query = $db->insert($entity_type)->fields($base_columns);
    $base_revision_insert_query = $db->insert("{$entity_type}_revision")->fields($revision_columns);
    $data_revision_insert_query = $db->insert("{$entity_type}_field_revision")->fields($data_revision_columns);
    $data_insert_query = $db->insert("{$entity_type}_field_data")->fields($data_columns);
    $delete_query = $db->delete("{$entity_type}_temp");

    $id = ((int) $db->query("SELECT MAX(id) FROM `{$entity_type}`")->fetchColumn()) + 1;
    $vid = ((int) $db->query("SELECT MAX(vid) FROM `{$entity_type}`")->fetchColumn()) + 1;

    foreach ($records as $object) {

      $time = time();

      $valid = TRUE;

      foreach ($schema['primary key'] as $key) {
        // Discard invalid objects.
        if ($object[$key] != 0 && empty($object[$key])) {
          $valid = FALSE;
        }
      }

      if (isset($schema['foreign keys'])) {
        foreach ($schema['foreign keys'] as $specification) {
          foreach ($specification['columns'] as $source_field => $destination_field) {

            $target_id = $db->query(
              "SELECT `{$destination_field}`
               FROM `{$specification['table']}_field_data`
               WHERE `{$source_field}` = '{$object[$source_field]}'
               AND `feed_reference__target_id` = :frti",
               [':frti' => $feedReference['target_id']]
            )->fetchColumn();

            // Sometimes, like with stops, the source field will be 0
            // which is semantically equal to blank but is valid.
            if (!$target_id && $object[$source_field] != 0) {
              $valid = FALSE;
            }
            else {
              $object[$source_field] = (
                $object[$source_field] === 0
                || $object[$source_field] === '0'
                ) ? 0 : $target_id;
            }
          }
        }
      }

      if (!$valid) {
        \Drupal::logger('Invalid import')->notice("<pre>" . print_r($object, true) . "</pre>");
        continue;
      }

      $object['id'] = $id;
      $object['vid'] = $vid;

      $uuid = (new UUID())->generate();
      $base_values = [$object['id'], $object['vid'], $uuid, 'en'];
      $base_insert_query->values(array_combine($base_columns, $base_values));

      $revision_values = [
        $object['id'],
        $object['vid'],
        'en',
        '1',
        '1',
        $time,
        '',
      ];

      $base_revision_insert_query->values(array_combine($revision_columns, $revision_values));

      $data_revision_insert_query->values($object);

      $object['created'] = $object['changed'] = $time;

      $data_insert_query->values($object);

      $id++;
      $vid++;
      $count++;
    }

    $base_insert_query->execute();
    $base_revision_insert_query->execute();
    $data_insert_query->execute();
    $data_revision_insert_query->execute();
    $delete_query
      ->condition('index_hash', array_column($records, 'index_hash'), 'IN')
      ->execute();

    Drupal::lock()->release('gtfs_lock');

    $context['message'] = t('Inserted @count new @tablename records', ['@count' => $count, '@tablename' => $entity_type]);
    if (isset($context['results']['inserted'])) {
      $context['results']['inserted'] += $count;
    }
    else {
      $context['results']['inserted'] = $count;
    }
  }

  /**
   * Transforms and updates a batch of entities.
   *
   * @param string $class
   *   The class from which to update entities.
   * @param array $context
   *   The batch context.
   *
   * @see https://www.drupal.org/docs/7/api/batch-api
   */
  public function transformUpdateExisting(string $class, array &$context) {
    $db = Drupal::database();
    $entity_type = \Drupal::service('entity_type.repository')->getEntityTypeFromClass($class);
    $limit = self::BATCH_SIZE;
    $feedReference = $this->getFeedReference();
    $count = 0;
    $existing_records = $db->query(
      "SELECT `fd`.`index_hash`
       FROM `{$entity_type}_field_data` `fd`
       WHERE `fd`.`index_hash` IN (
         SELECT `t`.`index_hash`
         FROM `{$entity_type}_temp` `t`
       ) AND `fd`.`hash` NOT IN (
         SELECT `t`.`hash`
         FROM `{$entity_type}_temp` `t`
       )
       LIMIT {$limit}"
    )->fetchAll(PDO::FETCH_COLUMN, 0);

    foreach ($existing_records as $index_hash) {
      $temp = $db->query(
        "SELECT *
         FROM `{$entity_type}_temp`
         WHERE `index_hash` = :index",
        [':index' => $index_hash]
      )->fetch(PDO::FETCH_ASSOC);

      $existing = $class::load($db->query(
        "SELECT `id`
         FROM `{$entity_type}_field_data`
         WHERE `index_hash` = :index",
        [':index' => $index_hash]
      )->fetchColumn());

      foreach ($temp as $key => $value) {
        if ($existing->{$key} != $value) {
          $existing->setNewRevision();
        }
        $existing->set($key, $value);
      }

      $existing->save();
      $count++;
    }

    $db->delete("{$entity_type}_temp")
      ->condition('index_hash', $existing_records, 'IN')
      ->execute();

    $context['message'] = t('Updated @count existing @tablename records', ['@count' => $count, '@tablename' => $entity_type]);
    if (isset($context['results']['updated'])) {
      $context['results']['updated'] += $count;
    }
    else {
      $context['results']['updated'] = $count;
    }

  }

  /**
   * Deletes entities not present in import.
   *
   * @param string $class
   *   The class from which to delete entities.
   * @param array $context
   *   The batch context.
   *
   * @see https://www.drupal.org/docs/7/api/batch-api
   */
  public function transformDeleteOld(string $class, array &$context) {
    $entity_type = \Drupal::service('entity_type.repository')->getEntityTypeFromClass($class);
    $limit = self::BATCH_SIZE;
    $feedReference = $this->getFeedReference();
    $count = 0;
    $old_records = Drupal::database()->query(
      "SELECT `fd`.`id`
       FROM `{$entity_type}_field_data` `fd`
       WHERE `fd`.`index_hash` NOT IN (
         SELECT `t`.`index_hash`
         FROM `{$entity_type}_temp` `t`
       )
       AND `fd`.`feed_reference__target_id` = :frti
       LIMIT {$limit}",
      [':frti' => $feedReference['target_id']]
    )->fetchAll(PDO::FETCH_COLUMN, 0);

    foreach ($old_records as $id) {
      $existing = $class::load($id);
      $existing->delete();
      $count++;
    }

    $context['message'] = t('Deleted @count old records', ['@count' => $count]);
    if (isset($context['results']['deleted'])) {
      $context['results']['deleted'] += $count;
    }
    else {
      $context['results']['deleted'] = $count;
    }
  }

  /**
   * Cleans up temporary tables after inerting a class.
   *
   * @param bool $success
   *   Whether or not the operation succeeded.
   * @param array $results
   *   Data stored during operation in $context['results'].
   * @param array $operations
   *   Failed operations.
   *
   * @todo something interesting with $results.
   *
   * @see https://www.drupal.org/docs/7/api/batch-api
   */
  public function batchTransformFinished(bool $success, array $results, array $operations = []) {
    if (!$success) {
      Drupal::logger('gtfs')->error("<pre>" . print_r($results, TRUE) . print_r($operations, TRUE) . "</pre>");
      return;

    }
    $completed = $this->completed;
    $files = $this->files;
    $file = current(array_diff($files, $completed));
    $file_entity_map = static::getFileEntityMap();
    $class = $file_entity_map[$file];
    $completed[] = $file;
    $this->completed = $completed;
    $temp_table_name = \Drupal::service('entity_type.repository')->getEntityTypeFromClass($class) . '_temp';
    $db = Drupal::database();
    $db_schema = $db->schema();
    $db_schema->dropTable($temp_table_name);
  }

  /**
   * Creates a batch to cleanup the import's mess.
   *
   * @return array
   *   Drupal batch description.
   *
   * @see https://www.drupal.org/docs/7/api/batch-api
   */
  public function batchCleanup(): array {
    $batch = [
      'title' => t('Cleaning up'),
      'operations' => [],
      'finished' => [$this, 'batchCleanupFinished'],
    ];
    $file_entity_map = static::getFileEntityMap();
    $downloader = new Downloader($this->feed);
    $db_schema = Drupal::database()->schema();
    $batch['operations'][] = [
      [$downloader, 'removeDirectory'],
    ];
    foreach ($file_entity_map as $file => $class) {
      $temp_table_name = \Drupal::service('entity_type.repository')->getEntityTypeFromClass($class) . '_temp';
      if ($db_schema->tableExists($temp_table_name)) {
        $db_schema->dropTable($temp_table_name);
      }
      if (method_exists($class, 'postImportBatchOperations')) {
        $batch['operations'] = array_merge($batch['operations'], $class::postImportBatchOperations());
      }
    }
    return $batch;
  }

  /**
   *
   */
  public function batchCleanupFinished($success, $results, $operations) {}

}


// https://stackoverflow.com/a/24927253
class EOLDetector {
  const EOL_UNIX    = 'lf';        // Code: \n
  const EOL_TRS80   = 'cr';        // Code: \r
  const EOL_ACORN   = 'lfcr';      // Code: \n \r
  const EOL_WINDOWS = 'crlf';      // Code: \r \n

  /**
  Detects the end-of-line character of a string.
  @param string $str      The string to check.
  @param string $key      [io] Name of the detected eol key.
  @return string The detected EOL, or default one.
  */
  public static function detectEOL($str, &$key) {
     static $eols = array(
       self::EOL_ACORN   => "\n\r",  // 0x0A - 0x0D - acorn BBC
       self::EOL_WINDOWS => "\r\n",  // 0x0D - 0x0A - Windows, DOS OS/2
       self::EOL_UNIX    => "\n",    // 0x0A -      - Unix, OSX
       self::EOL_TRS80   => "\r",    // 0x0D -      - Apple ][, TRS80
    );

    $key = "";
    $curCount = 0;
    $curEol = '';
    foreach($eols as $k => $eol) {
       if( ($count = substr_count($str, $eol)) > $curCount) {
          $curCount = $count;
          $curEol = $eol;
          $key = $k;
       }
    }
    return $curEol;
  }

  /**
  Detects the EOL of an file by checking the first line.
  @param string  $fileName    File to be tested (full pathname).
  @return boolean false | Used key = enum('cr', 'lf', crlf').
  @uses detectEOL
  */
  public static function detectFileEOL($fileName) {
     if (!file_exists($fileName)) {
       return false;
     }

     // Gets the line length
     $handle = @fopen($fileName, "r");
     if ($handle === false) {
        return false;
     }
     $line = fgets($handle);
     $key = "";
     return self::detectEOL($line, $key);

     return $key;
  }  // detectFileEOL
}
