<?php

namespace Drupal\gtfs\Entity;

use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\RevisionableContentEntityBase;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\RevisionableInterface;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\user\UserInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\file\Entity\File;

/**
 * This class defines generic functions as required by
 * the interface, except for those containing crucial
 * data like getGTFSFields. These functions should be
 * adequate for basic GTFS objects like Agency but will
 * be overwritten for more complex ones like StopTimes.
 * Inheriting objects may use these parent functions as
 * a starting point.
 */
abstract class GTFSEntityBase extends RevisionableContentEntityBase implements GTFSObjectInterface {

  use EntityChangedTrait;

  /**
   * This gets set when updating from a feed.
   */
  public $isChangedOrNew = FALSE;

  public static $resourceURL = '/gtfs/api/v1';

  /**
   * The Drupal fields we don't want output.
   *
   * @var array
   */
  public static $fields_to_hide = [
    'id',
    'uuid',
    'vid',
    'hash',
    'index_hash',
    'created',
    'changed',
    'langcode',
    'revision_created',
    'revision_user',
    'revision_log_message',
    'default_langcode',
    'revision_default',
    'user_id',
    'status',
    'revision_translation_affected',
    'feed_reference',
  ];

  /**
   *
   */
  public function toGTFSObject() {
    $fields = array_diff(array_keys(static::getFields()), static::$fields_to_hide, self::$fields_to_hide);
    $object = array_combine($fields, array_map(function ($field) {
      $definition = static::getFieldDefinition($field);
      $type = $definition->getType();
      switch ($type) {
        case 'entity_reference':
          $target_type = $definition->getSetting('target_type');
          $id = $this->get($field)->target_id;
          return \Drupal::database()->query(
            "SELECT {$field}
             FROM {{$target_type}_field_data}
             WHERE id = :id",
             [':id' => $id]
          )->fetchColumn();
          break;
        case 'file':
        case 'image':
          $obj = $this->get($field)->get(0);
          return $obj ? file_create_url(File::load($obj->target_id)->getFileUri()) : '';
          break;
        default:
          return $this->get($field)->value;
          break;
      }
    }, $fields));
    $object['self'] = $this->resourceUrl();
    return $object;
  }

  /**
   *
   */
  public function toGTFSRow() {
    return implode(',', array_values($this->toGTFSObject()));
  }

  /**
   *
   */
  public static function getUniqueIdField() {
    return FALSE;
  }

  /**
   *
   */
  public function resourceUrl() {
    $url = \Drupal::request()->getSchemeAndHttpHost() . static::$resourceURL;
    $schema = static::schema();
    if (isset($schema['primary key'])) {
      foreach ($schema['primary key'] as $key) {
        $url .= "/{$this->get($key)->value}";
      }
    }
    return $url;
  }

  /**
   *
   */
  public function links() {
    $entity_type = \Drupal::service('entity_type.repository')->getEntityTypeFromClass(get_called_class());

    $links = [
      'self' => $this->resourceUrl(),
    ];

    \Drupal::moduleHandler()->alter("{$entity_type}_links", $links, $this);

    return $links;
  }

  /**
   * BELOW THIS POINT all methods are inherited from RevisionableContentEntityBase
   * or its parents.
   */

  /**
   * {@inheritdoc}
   */
  public static function preCreate(EntityStorageInterface $storage_controller, array &$values) {
    parent::preCreate($storage_controller, $values);
    $values += [
      'user_id' => \Drupal::currentUser()->id(),
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function urlRouteParameters($rel) {
    $uri_route_parameters = parent::urlRouteParameters($rel);

    if ($rel === 'revision_revert' && $this instanceof RevisionableInterface) {
      $uri_route_parameters[$this->getEntityTypeId() . '_revision'] = $this->getRevisionId();
    }
    elseif ($rel === 'revision_delete' && $this instanceof RevisionableInterface) {
      $uri_route_parameters[$this->getEntityTypeId() . '_revision'] = $this->getRevisionId();
    }

    return $uri_route_parameters;
  }

  /**
   * {@inheritdoc}
   */
  public function preSave(EntityStorageInterface $storage) {
    parent::preSave($storage);

    foreach (array_keys($this->getTranslationLanguages()) as $langcode) {
      $translation = $this->getTranslation($langcode);

      // If no owner has been set explicitly, make the anonymous user the owner.
      if (!$translation->getOwner()) {
        $translation->setOwnerId(0);
      }
    }

    // If no revision author has been set explicitly, make the agency owner the
    // revision author.
    if (!$this->getRevisionUser()) {
      $this->setRevisionUserId($this->getOwnerId());
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($timestamp) {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwner() {
    return $this->get('user_id')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwnerId() {
    return $this->get('user_id')->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwnerId($uid) {
    $this->set('user_id', $uid);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwner(UserInterface $account) {
    $this->set('user_id', $account->id());
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function isPublished() {
    return (bool) $this->getEntityKey('status');
  }

  /**
   * {@inheritdoc}
   */
  public function setPublished($published) {
    $this->set('status', $published ? TRUE : FALSE);
    return $this;
  }

  /**
   * Set up some fields common to all our GTFS entities,
   * including a link back to the feed to which they belong.
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['user_id'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Authored by'))
      ->setDescription(t('The user ID of author of the GTFS Feed entity.'))
      ->setRevisionable(TRUE)
      ->setSetting('target_type', 'user')
      ->setSetting('handler', 'default')
      ->setTranslatable(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'author',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 5,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['feed_reference'] = BaseFieldDefinition::create('entity_reference_revisions')
      ->setLabel(t('Parent Feed'))
      ->setDescription(t('The dataset to which the entity is attached.'))
      ->setRevisionable(TRUE)
      ->setSettings([
        'target_type' => 'gtfs_feed',
        'target_bundle' => 'gtfs_feed',
        'default_value' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_revisions_autocomplete"',
        'weight' => -3,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setRequired(TRUE);

    $fields['hash'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Hash of import data'))
      ->setRevisionable(TRUE);

    $fields['index_hash'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Hash of unique indexes'))
      ->setRevisionable(TRUE);

    $fields['status'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Publishing status'))
      ->setDescription(t('A boolean indicating whether the entity is published.'))
      ->setRevisionable(TRUE)
      ->setDefaultValue(TRUE)
      ->setDisplayOptions('form', [
        'type' => 'boolean_checkbox',
        'weight' => 4,
      ]);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created'))
      ->setDescription(t('The time that the entity was created.'));

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the entity was last edited.'));

    $fields['revision_translation_affected'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Revision translation affected'))
      ->setDescription(t('Indicates if the last edit of a translation belongs to current revision.'))
      ->setReadOnly(TRUE)
      ->setRevisionable(TRUE)
      ->setTranslatable(TRUE);

    return $fields;
  }

}
